@@ -1,7 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 /*
- * arch-tag: Interface to audio encoder backend
- *
  * Copyright (C) 2006 James Livingston <doclivingston@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -33,7 +31,7 @@
 
 #include <glib-object.h>
 
-#include "rhythmdb.h"
+#include <rhythmdb/rhythmdb.h>
 
 G_BEGIN_DECLS
 
@@ -42,58 +40,100 @@ G_BEGIN_DECLS
 #define RB_IS_ENCODER(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RB_TYPE_ENCODER))
 #define RB_ENCODER_GET_IFACE(obj)  (G_TYPE_INSTANCE_GET_INTERFACE ((obj), RB_TYPE_ENCODER, RBEncoderIface))
 
+#define RB_TYPE_ENCODER_FACTORY        (rb_encoder_factory_get_type ())
+#define RB_ENCODER_FACTORY(obj)        (G_TYPE_CHECK_INSTANCE_CAST ((obj), RB_TYPE_ENCODER, RBEncoderFactory))
+#define RB_ENCODER_FACTORY_CLASS(k)    (G_TYPE_CHECK_CLASS_CAST((k), RB_TYPE_ENCODER_FACTORY, RBEncoderFactoryClass))
+#define RB_IS_ENCODER_FACTORY(obj)     (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RB_TYPE_ENCODER))
+#define RB_IS_ENCODER_FACTORY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), RB_TYPE_ENCODER_FACTORY))
+#define RB_ENCODER_FACTORY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), RB_TYPE_ENCODER_FACTORY, RBEncoderFactoryClass))
+
 enum
 {
 	RB_ENCODER_ERROR_FORMAT_UNSUPPORTED,
 	RB_ENCODER_ERROR_INTERNAL,
 	RB_ENCODER_ERROR_FILE_ACCESS,
+	RB_ENCODER_ERROR_OUT_OF_SPACE,
+	RB_ENCODER_ERROR_DEST_READ_ONLY
 };
 
+GType rb_encoder_error_get_type (void);
+#define RB_TYPE_ENCODER_ERROR	(rb_encoer_error_get_type())
 #define RB_ENCODER_ERROR rb_encoder_error_quark ()
-
 GQuark rb_encoder_error_quark (void);
 
 typedef struct _RBEncoder RBEncoder;
+typedef struct _RBEncoderIface RBEncoderIface;
+typedef struct _RBEncoderFactory RBEncoderFactory;
+typedef struct _RBEncoderFactoryClass RBEncoderFactoryClass;
 
-typedef struct
+struct _RBEncoderIface
 {
 	GTypeInterface g_iface;
 
 	/* vtable */
-	gboolean	(*encode)	(RBEncoder *encoder,
+	void		(*encode)	(RBEncoder *encoder,
 					 RhythmDBEntry *entry,
 					 const char *dest,
-					 GList *mime_types);
+					 const char *dest_media_type);
 	void		(*cancel)	(RBEncoder *encoder);
-	gboolean	(*get_preferred_mimetype)	(RBEncoder *encoder,
-							 GList *mime_types,
-							 char **mime,
-							 char **extension);
+	gboolean	(*get_media_type) (RBEncoder *encoder,
+					 RhythmDBEntry *entry,
+					 GList *dest_media_types,
+					 char **media_type,
+					 char **extension);
+	gboolean	(*get_missing_plugins) (RBEncoder *encoder,
+					 const char *media_type,
+					 char ***details);
 
 	/* signals */
 	void (*progress) (RBEncoder *encoder,  double fraction);
-	void (*completed) (RBEncoder *encoder, guint64 dest_size);
-	void (*error) (RBEncoder *encoder, GError *error);
-} RBEncoderIface;
+	gboolean (*overwrite) (RBEncoder *encoder, GFile *file);
+	void (*completed) (RBEncoder *encoder, guint64 dest_size, const char *mediatype, GError *error);
+};
+
+struct _RBEncoderFactoryClass
+{
+	GObjectClass obj_class;
+
+	/* signals */
+	void (*prepare_source) (RBEncoderFactory *factory, const char *uri, GObject *source);
+	void (*prepare_sink) (RBEncoderFactory *factory, const char *uri, GObject *sink);
+};
+
+struct _RBEncoderFactory
+{
+	GObject obj;
+};
+
+GType 		rb_encoder_factory_get_type (void);
+RBEncoderFactory *rb_encoder_factory_get (void);
+
 
 RBEncoder*	rb_encoder_new		(void);
-GType rb_encoder_get_type (void);
+GType 		rb_encoder_get_type 	(void);
 
-gboolean	rb_encoder_encode	(RBEncoder *encoder,
+void		rb_encoder_encode	(RBEncoder *encoder,
 					 RhythmDBEntry *entry,
 					 const char *dest,
-					 GList *mime_types);
+					 const char *dest_media_type);
 void		rb_encoder_cancel	(RBEncoder *encoder);
 
-gboolean	rb_encoder_get_preferred_mimetype (RBEncoder *encoder,
-						   GList *mime_types,
-						   char **mime,
-						   char **extension);
+gboolean	rb_encoder_get_media_type (RBEncoder *encoder,
+					 RhythmDBEntry *entry,
+					 GList *dest_media_types,
+					 char **media_type,
+					 char **extension);
+gboolean	rb_encoder_get_missing_plugins (RBEncoder *encoder,
+					 const char *media_type,
+					 char ***details);
 
 /* only to be used by subclasses */
 void	_rb_encoder_emit_progress (RBEncoder *encoder, double fraction);
-void	_rb_encoder_emit_completed (RBEncoder *encoder, guint64 dest_size);
-void	_rb_encoder_emit_error (RBEncoder *encoder, GError *error);
+void	_rb_encoder_emit_completed (RBEncoder *encoder, guint64 dest_size, const char *mediatype, GError *error);
+gboolean _rb_encoder_emit_overwrite (RBEncoder *encoder, GFile *file);
+
+void	_rb_encoder_emit_prepare_source (RBEncoder *encoder, const char *uri, GObject *source);
+void	_rb_encoder_emit_prepare_sink (RBEncoder *encoder, const char *uri, GObject *sink);
 
 G_END_DECLS
 
