@@ -33,6 +33,21 @@
 #include "rb-player-gst-xfade.h"
 #include "rb-marshal.h"
 
+/**
+ * RBPlayerPlayType:
+ * @RB_PLAYER_PLAY_REPLACE: Replace the existing stream
+ * @RB_PLAYER_PLAY_AFTER_EOS: Start the new stream after the current stream ends
+ * @RB_PLAYER_PLAY_CROSSFADE: Crossfade between the existing stream and the new stream
+ */
+
+/**
+ * RBPlayerError:
+ * @RB_PLAYER_ERROR_NO_AUDIO: Audio playback not available
+ * @RB_PLAYER_ERROR_GENERAL: Nonspecific error
+ * @RB_PLAYER_ERROR_INTERNAL: Internal error
+ * @RB_PLAYER_ERROR_NOT_FOUND: The resource could not be found
+ */
+
 /* Signals */
 enum {
 	EOS,
@@ -44,6 +59,7 @@ enum {
 	PLAYING_STREAM,
 	VOLUME_CHANGED,
 	IMAGE,
+	REDIRECT,
 	LAST_SIGNAL
 };
 
@@ -274,6 +290,25 @@ rb_player_interface_init (RBPlayerIface *iface)
 			      G_TYPE_NONE,
 			      2,
 			      G_TYPE_POINTER, GDK_TYPE_PIXBUF);
+
+	/**
+	 * RBPlayer::redirect:
+	 * @player: the #RBPlayer
+	 * @stream_data: data associated with the stream
+	 * @uri: URI to redirect to
+	 *
+	 * The 'redirect' signal is emitted to indicate when a stream has change URI.
+	 */
+	signals[REDIRECT] =
+		g_signal_new ("redirect",
+			      G_TYPE_FROM_INTERFACE (iface),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RBPlayerIface, redirect),
+			      NULL, NULL,
+			      rb_marshal_VOID__POINTER_STRING,
+			      G_TYPE_NONE,
+			      2,
+			      G_TYPE_POINTER, G_TYPE_STRING);
 }
 
 GType
@@ -691,6 +726,21 @@ _rb_player_emit_image (RBPlayer *player, gpointer stream_data, GdkPixbuf *image)
 	g_signal_emit (player, signals[IMAGE], 0, stream_data, image);
 }
 
+/**
+ * _rb_player_emit_redirect:
+ * @player: a #RBPlayer implementation
+ * @stream_data: data associated with the stream
+ * @uri: URI to redirect to
+ *
+ * Emits the 'redirect' signal to notify listeners that the stream has been
+ * redirected. To be used by implementations only.
+ */
+void
+_rb_player_emit_redirect (RBPlayer *player, gpointer stream_data, const char *uri)
+{
+	g_signal_emit (player, signals[REDIRECT], 0, stream_data, uri);
+}
+
 GQuark
 rb_player_error_quark (void)
 {
@@ -711,9 +761,10 @@ rb_player_error_get_type (void)
 
 	if (etype == 0)	{
 		static const GEnumValue values[] = {
-			ENUM_ENTRY (RB_PLAYER_ERROR_NO_AUDIO, "Stream contains no audio"),
-			ENUM_ENTRY (RB_PLAYER_ERROR_GENERAL, "General error"),
-			ENUM_ENTRY (RB_PLAYER_ERROR_INTERNAL, "Internal backend error"),
+			ENUM_ENTRY (RB_PLAYER_ERROR_NO_AUDIO, "no-audio"),
+			ENUM_ENTRY (RB_PLAYER_ERROR_GENERAL, "general-error"),
+			ENUM_ENTRY (RB_PLAYER_ERROR_INTERNAL, "internal-error"),
+			ENUM_ENTRY (RB_PLAYER_ERROR_NOT_FOUND, "not-found"),
 			{ 0, 0, 0 }
 		};
 
@@ -730,9 +781,9 @@ rb_player_play_type_get_type (void)
 
 	if (etype == 0)	{
 		static const GEnumValue values[] = {
-			ENUM_ENTRY (RB_PLAYER_PLAY_REPLACE, "Replace existing stream"),
-			ENUM_ENTRY (RB_PLAYER_PLAY_AFTER_EOS, "Start new stream after EOS of existing stream"),
-			ENUM_ENTRY (RB_PLAYER_PLAY_CROSSFADE, "Crossfade between streams"),
+			ENUM_ENTRY (RB_PLAYER_PLAY_REPLACE, "replace"),
+			ENUM_ENTRY (RB_PLAYER_PLAY_AFTER_EOS, "start-after-eos"),
+			ENUM_ENTRY (RB_PLAYER_PLAY_CROSSFADE, "crossfade"),
 			{ 0, 0, 0 }
 		};
 
