@@ -72,7 +72,6 @@
 #include "rb-dialog.h"
 #include "rb-debug.h"
 #include "eel-gconf-extensions.h"
-#include "gedit-message-area.h"
 #include "rb-shell-player.h"
 #include "rb-play-order.h"
 #include "rb-lastfm-play-order.h"
@@ -120,7 +119,6 @@ static void queue_get_playlist_and_skip (RBLastfmSource *source, RhythmDBEntry *
 static void queue_love_track (RBLastfmSource *source);
 static void queue_ban_track (RBLastfmSource *source);
 
-
 static void rb_lastfm_source_class_init (RBLastfmSourceClass *klass);
 static void rb_lastfm_source_init (RBLastfmSource *source);
 static void rb_lastfm_source_constructed (GObject *object);
@@ -161,7 +159,13 @@ static gboolean impl_receive_drag (RBSource *source, GtkSelectionData *data);
 static void impl_activate (RBSource *source);
 static gboolean impl_show_popup (RBSource *source);
 static guint impl_want_uri (RBSource *source, const char *uri);
-static gboolean impl_add_uri (RBSource *source, const char *uri, const char *title, const char *genre);
+static gboolean impl_add_uri (RBSource *source,
+			      const char *uri,
+			      const char *title,
+			      const char *genre,
+			      RBSourceAddCallback callback,
+			      gpointer data,
+			      GDestroyNotify destroy_data);
 static RBSourceEOFType impl_handle_eos (RBSource *asource);
 
 static void rb_lastfm_source_new_station (const char *uri, const char *title, RBLastfmSource *source);
@@ -208,7 +212,6 @@ typedef struct
 	char *download_url;
 } RBLastfmTrackEntryData;
 
-
 struct RBLastfmSourcePrivate
 {
 	GtkWidget *main_box;
@@ -225,8 +228,8 @@ struct RBLastfmSourcePrivate
 	RBEntryView *tracks;
 
 	RBShellPlayer *shell_player;
-	RhythmDBEntryType station_entry_type;
-	RhythmDBEntryType track_entry_type;
+	RhythmDBEntryType *station_entry_type;
+	RhythmDBEntryType *track_entry_type;
 	char *session_id;
 	RhythmDBEntry *current_station;
 	RBPlayOrder *play_order;
@@ -329,18 +332,18 @@ rb_lastfm_source_class_init (RBLastfmSourceClass *klass)
 
 	g_object_class_install_property (object_class,
 					 PROP_ENTRY_TYPE,
-					 g_param_spec_boxed ("entry-type",
-							     "Entry type",
-							     "Entry type for last.fm tracks",
-							     RHYTHMDB_TYPE_ENTRY_TYPE,
-							     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+					 g_param_spec_object ("entry-type",
+							      "Entry type",
+							      "Entry type for last.fm tracks",
+							      RHYTHMDB_TYPE_ENTRY_TYPE,
+							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class,
 					 PROP_STATION_ENTRY_TYPE,
-					 g_param_spec_boxed ("station-entry-type",
-							     "Entry type",
-							     "Entry type for last.fm stations",
-							     RHYTHMDB_TYPE_ENTRY_TYPE,
-							     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+					 g_param_spec_object ("station-entry-type",
+							      "Entry type",
+							      "Entry type for last.fm stations",
+							      RHYTHMDB_TYPE_ENTRY_TYPE,
+							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_override_property (object_class,
 					  PROP_PLAY_ORDER,
 					  "play-order");
@@ -618,10 +621,10 @@ rb_lastfm_source_set_property (GObject *object,
 
 	switch (prop_id) {
 	case PROP_ENTRY_TYPE:
-		source->priv->track_entry_type = g_value_get_boxed (value);
+		source->priv->track_entry_type = g_value_get_object (value);
 		break;
 	case PROP_STATION_ENTRY_TYPE:
-		source->priv->station_entry_type = g_value_get_boxed (value);
+		source->priv->station_entry_type = g_value_get_object (value);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -639,10 +642,10 @@ rb_lastfm_source_get_property (GObject *object,
 
 	switch (prop_id) {
 	case PROP_ENTRY_TYPE:
-		g_value_set_boxed (value, source->priv->track_entry_type);
+		g_value_set_object (value, source->priv->track_entry_type);
 		break;
 	case PROP_STATION_ENTRY_TYPE:
-		g_value_set_boxed (value, source->priv->station_entry_type);
+		g_value_set_object (value, source->priv->station_entry_type);
 		break;
 	case PROP_PLAY_ORDER:
 		g_value_set_object (value, source->priv->play_order);
@@ -653,10 +656,8 @@ rb_lastfm_source_get_property (GObject *object,
 	}
 }
 
-/* entry data stuff */
-
 static void
-destroy_track_data (RhythmDBEntry *entry, gpointer meh)
+destroy_track_data (RhythmDBEntryType *etype, RhythmDBEntry *entry)
 {
 	RBLastfmTrackEntryData *data;
 
@@ -672,31 +673,38 @@ rb_lastfm_source_new (RBPlugin *plugin,
 		      RBShell  *shell)
 {
 	RBSource *source;
-	RhythmDBEntryType station_entry_type;
-	RhythmDBEntryType track_entry_type;
+	RhythmDBEntryType *station_entry_type;
+	RhythmDBEntryType *track_entry_type;
 	RhythmDB *db;
 
 	g_object_get (G_OBJECT (shell), "db", &db, NULL);
 
 	/* register entry types if they're not already registered */
 	station_entry_type = rhythmdb_entry_type_get_by_name (db, "lastfm-station");
-	if (station_entry_type == RHYTHMDB_ENTRY_TYPE_INVALID) {
-		station_entry_type = rhythmdb_entry_register_type (db, "lastfm-station");
-		station_entry_type->save_to_disk = TRUE;
-		station_entry_type->can_sync_metadata = (RhythmDBEntryCanSyncFunc) rb_true_function;
-		station_entry_type->sync_metadata = (RhythmDBEntrySyncFunc) rb_null_function;
-		station_entry_type->get_playback_uri = (RhythmDBEntryStringFunc) rb_null_function;	/* can't play stations, exactly */
-		station_entry_type->category = RHYTHMDB_ENTRY_CONTAINER;
+	if (station_entry_type == NULL) {
+		station_entry_type = g_object_new (RHYTHMDB_TYPE_ENTRY_TYPE,
+						   "db", db,
+						   "name", "lastfm-station",
+						   "save-to-disk", TRUE,
+						   "category", RHYTHMDB_ENTRY_CONTAINER,
+						   NULL);
+		station_entry_type->can_sync_metadata = (RhythmDBEntryTypeBooleanFunc) rb_true_function;
+		station_entry_type->sync_metadata = (RhythmDBEntryTypeSyncFunc) rb_null_function;
+		station_entry_type->get_playback_uri = (RhythmDBEntryTypeStringFunc) rb_null_function;
+		rhythmdb_register_entry_type (db, station_entry_type);
 	}
 
 	track_entry_type = rhythmdb_entry_type_get_by_name (db, "lastfm-track");
-	if (track_entry_type == RHYTHMDB_ENTRY_TYPE_INVALID) {
-		track_entry_type = rhythmdb_entry_register_type (db, "lastfm-track");
-		track_entry_type->save_to_disk = FALSE;
-		track_entry_type->category = RHYTHMDB_ENTRY_NORMAL;
-
-		track_entry_type->entry_type_data_size = sizeof (RBLastfmTrackEntryData);
-		track_entry_type->pre_entry_destroy = destroy_track_data;
+	if (track_entry_type == NULL) {
+		track_entry_type = g_object_new (RHYTHMDB_TYPE_ENTRY_TYPE,
+						 "db", db,
+						 "name", "lastfm-track",
+						 "save-to-disk", FALSE,
+						 "category", RHYTHMDB_ENTRY_NORMAL,
+						 "type-data-size", sizeof (RBLastfmTrackEntryData),
+						 NULL);
+		track_entry_type->destroy_entry = destroy_track_data;
+		rhythmdb_register_entry_type (db, track_entry_type);
 	}
 
 	source = RB_SOURCE (g_object_new (RB_TYPE_LASTFM_SOURCE,
@@ -746,6 +754,7 @@ set_message_area_text_and_icon (RBLastfmSource *source,
 	char      *secondary_markup;
 	GtkWidget *primary_label;
 	GtkWidget *secondary_label;
+	GtkWidget *content;
 
 	hbox_content = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (hbox_content);
@@ -785,8 +794,8 @@ set_message_area_text_and_icon (RBLastfmSource *source,
 	}
 
 	gtk_widget_show (source->priv->message_area);
-	gedit_message_area_set_contents (GEDIT_MESSAGE_AREA (source->priv->message_area),
-					 hbox_content);
+	content = gtk_info_bar_get_content_area (GTK_INFO_BAR (source->priv->message_area));
+	gtk_container_add (GTK_CONTAINER (content), hbox_content);
 }
 
 static void
@@ -818,9 +827,9 @@ set_message_area (RBLastfmSource *source,
 }
 
 static void
-on_message_area_response (GeditMessageArea *area,
-			  int               response_id,
-			  RBLastfmSource   *source)
+on_message_area_response (GtkInfoBar     *infobar,
+			  int             response_id,
+			  RBLastfmSource *source)
 {
 	RBPlugin  *plugin;
 	GtkWidget *dialog;
@@ -841,9 +850,9 @@ show_error_message (RBLastfmSource *source,
 		return;
 	}
 
-	area = gedit_message_area_new_with_buttons (_("Account Settings"),
-						    GTK_RESPONSE_ACCEPT,
-						    NULL);
+	area = gtk_info_bar_new_with_buttons (_("Account Settings"),
+					      GTK_RESPONSE_ACCEPT,
+					      NULL);
 	set_message_area (source, area);
 	set_message_area_text_and_icon (source,
 					"gtk-dialog-error",
@@ -1394,17 +1403,29 @@ impl_want_uri (RBSource *source, const char *uri)
 }
 
 static gboolean
-impl_add_uri (RBSource *source, const char *uri, const char *title, const char *genre)
+impl_add_uri (RBSource *source,
+	      const char *uri,
+	      const char *title,
+	      const char *genre,
+	      RBSourceAddCallback callback,
+	      gpointer data,
+	      GDestroyNotify destroy_data)
 {
 	char *name;
+	gboolean ret = FALSE;
 
-	if (strstr (uri, "lastfm://") == NULL)
-		return FALSE;
+	if (strstr (uri, "lastfm://") != NULL) {
+		name = rb_lastfm_source_title_from_uri (uri);
+		rb_lastfm_source_new_station (uri, name, RB_LASTFM_SOURCE (source));
+	}
 
-	name = rb_lastfm_source_title_from_uri (uri);
-
-	rb_lastfm_source_new_station (uri, name, RB_LASTFM_SOURCE (source));
-	return TRUE;
+	if (callback != NULL) {
+		callback (source, uri, data);
+		if (destroy_data != NULL) {
+			destroy_data (data);
+		}
+	}
+	return ret;
 }
 
 static RBSourceEOFType
