@@ -1,6 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- *
- *  arch-tag: Implementation of Internet Radio source object
  *
  *  Copyright (C) 2002,2003 Colin Walters <walters@debian.org>
  *
@@ -34,14 +32,12 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 #include <libxml/tree.h>
 
 #include "rb-iradio-source.h"
 #include "rb-iradio-source-search.h"
 
 #include "rhythmdb-query-model.h"
-#include "rb-glade-helpers.h"
 #include "rb-stock-icons.h"
 #include "rb-entry-view.h"
 #include "rb-property-view.h"
@@ -67,8 +63,7 @@
 
 static void rb_iradio_source_class_init (RBIRadioSourceClass *klass);
 static void rb_iradio_source_init (RBIRadioSource *source);
-static GObject *rb_iradio_source_constructor (GType type, guint n_construct_properties,
-					      GObjectConstructParam *construct_properties);
+static void rb_iradio_source_constructed (GObject *object);
 static void rb_iradio_source_dispose (GObject *object);
 static void rb_iradio_source_set_property (GObject *object,
 			                  guint prop_id,
@@ -111,7 +106,13 @@ static void impl_song_properties (RBSource *source);
 static gboolean impl_show_popup (RBSource *source);
 static GList *impl_get_ui_actions (RBSource *source);
 static guint impl_want_uri (RBSource *source, const char *uri);
-static gboolean impl_add_uri (RBSource *source, const char *uri, const char *title, const char *genre);
+static gboolean impl_add_uri (RBSource *source,
+			      const char *uri,
+			      const char *title,
+			      const char *genre,
+			      RBSourceAddCallback callback,
+			      gpointer data,
+			      GDestroyNotify destroy_data);
 
 static void rb_iradio_source_do_query (RBIRadioSource *source);
 
@@ -191,7 +192,7 @@ rb_iradio_source_class_init (RBIRadioSourceClass *klass)
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
 
 	object_class->dispose = rb_iradio_source_dispose;
-	object_class->constructor = rb_iradio_source_constructor;
+	object_class->constructed = rb_iradio_source_constructed;
 
 	object_class->set_property = rb_iradio_source_set_property;
 	object_class->get_property = rb_iradio_source_get_property;
@@ -282,22 +283,20 @@ rb_iradio_source_dispose (GObject *object)
 	G_OBJECT_CLASS (rb_iradio_source_parent_class)->dispose (object);
 }
 
-static GObject *
-rb_iradio_source_constructor (GType type,
-			      guint n_construct_properties,
-			      GObjectConstructParam *construct_properties)
+static void
+rb_iradio_source_constructed (GObject *object)
 {
 	RBIRadioSource *source;
 	RBShell *shell;
 	GtkAction *action;
 
-	source = RB_IRADIO_SOURCE (G_OBJECT_CLASS (rb_iradio_source_parent_class)
-			->constructor (type, n_construct_properties, construct_properties));
+	RB_CHAIN_GOBJECT_METHOD (rb_iradio_source_parent_class, constructed, object);
+	source = RB_IRADIO_SOURCE (object);
 
 	source->priv->paned = gtk_hpaned_new ();
 
-	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
-	g_object_get (G_OBJECT (shell),
+	g_object_get (source, "shell", &shell, NULL);
+	g_object_get (shell,
 		      "db", &source->priv->db,
 		      "shell-player", &source->priv->player,
 		      NULL);
@@ -397,8 +396,6 @@ rb_iradio_source_constructor (GType type,
 	source->priv->default_search = rb_iradio_source_search_new ();
 
 	rb_iradio_source_do_query (source);
-
-	return G_OBJECT (source);
 }
 
 static void
@@ -435,18 +432,22 @@ RBSource *
 rb_iradio_source_new (RBShell *shell, RBPlugin *plugin)
 {
 	RBSource *source;
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 	RhythmDB *db;
 
 	g_object_get (shell, "db", &db, NULL);
 
 	entry_type = rhythmdb_entry_type_get_by_name (db, "iradio");
-	if (entry_type == RHYTHMDB_ENTRY_TYPE_INVALID) {
-		entry_type = rhythmdb_entry_register_type (db, "iradio");
-		entry_type->save_to_disk = TRUE;
-		entry_type->category = RHYTHMDB_ENTRY_STREAM;
-		entry_type->can_sync_metadata = (RhythmDBEntryCanSyncFunc) rb_true_function;
-		entry_type->sync_metadata = (RhythmDBEntrySyncFunc) rb_null_function;
+	if (entry_type == NULL) {
+		entry_type = g_object_new (RHYTHMDB_TYPE_ENTRY_TYPE,
+					   "db", db,
+					   "name", "iradio",
+					   "save-to-disk", TRUE,
+					   "category", RHYTHMDB_ENTRY_STREAM,
+					   NULL);
+		entry_type->can_sync_metadata = (RhythmDBEntryTypeBooleanFunc) rb_true_function;
+		entry_type->sync_metadata = (RhythmDBEntryTypeSyncFunc) rb_null_function;
+		rhythmdb_register_entry_type (db, entry_type);
 	}
 	g_object_unref (db);
 
@@ -493,7 +494,7 @@ rb_iradio_source_add_station (RBIRadioSource *source,
 	char *real_uri = NULL;
 	char *fixed_title;
 	char *fixed_genre = NULL;
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 
 	real_uri = guess_uri_scheme (uri);
 	if (real_uri)
@@ -508,7 +509,7 @@ rb_iradio_source_add_station (RBIRadioSource *source,
 
 	g_object_get (source, "entry-type", &entry_type, NULL);
 	entry = rhythmdb_entry_new (source->priv->db, entry_type, uri);
-	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+	g_object_unref (entry_type);
 	if (entry == NULL) {
 		g_free (real_uri);
 		return;
@@ -656,7 +657,13 @@ impl_want_uri (RBSource *source, const char *uri)
 }
 
 static gboolean
-impl_add_uri (RBSource *source, const char *uri, const char *title, const char *genre)
+impl_add_uri (RBSource *source,
+	      const char *uri,
+	      const char *title,
+	      const char *genre,
+	      RBSourceAddCallback callback,
+	      gpointer data,
+	      GDestroyNotify destroy_data)
 {
 	if (rb_uri_is_local (uri)) {
 		rb_iradio_source_add_from_playlist (RB_IRADIO_SOURCE (source), uri);
@@ -664,6 +671,12 @@ impl_add_uri (RBSource *source, const char *uri, const char *title, const char *
 		rb_iradio_source_add_station (RB_IRADIO_SOURCE (source),
 					      uri, title, genre);
 	}
+	if (callback != NULL) {
+		callback (source, uri, data);
+		if (destroy_data != NULL) {
+			destroy_data (data);
+		}
+	}
 	return TRUE;
 }
 
@@ -771,7 +784,7 @@ rb_iradio_source_do_query (RBIRadioSource *source)
 	RhythmDBQueryModel *station_query_model = NULL;
 	RhythmDBPropertyModel *genre_model;
 	GPtrArray *query;
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 
 	/* don't update the selection while we're rebuilding the query */
 	source->priv->setting_new_query = TRUE;
@@ -786,7 +799,7 @@ rb_iradio_source_do_query (RBIRadioSource *source)
 				      RHYTHMDB_PROP_TYPE,
 				      entry_type,
 				      RHYTHMDB_QUERY_END);
-	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+	g_object_unref (entry_type);
 
 	if (source->priv->search_query != NULL) {
 		rhythmdb_query_append (source->priv->db,
@@ -895,6 +908,7 @@ rb_iradio_source_add_from_playlist (RBIRadioSource *source,
 		rb_iradio_source_add_station (source, uri, NULL, NULL);
 		break;
 
+	default:
 	case TOTEM_PL_PARSER_RESULT_SUCCESS:
 	case TOTEM_PL_PARSER_RESULT_ERROR:
 		break;
@@ -947,7 +961,7 @@ stations_view_drag_data_received_cb (GtkWidget *widget,
 	GList *uri_list, *i;
 
 	rb_debug ("parsing uri list");
-	uri_list = rb_uri_list_parse ((char *)selection_data->data);
+	uri_list = rb_uri_list_parse ((char *) gtk_selection_data_get_data (selection_data));
 	if (uri_list == NULL)
 		return;
 
@@ -1015,13 +1029,13 @@ rb_iradio_source_cmd_new_station (GtkAction *action,
 static gboolean
 check_entry_type (RBIRadioSource *source, RhythmDBEntry *entry)
 {
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 	gboolean matches = FALSE;
 
 	g_object_get (source, "entry-type", &entry_type, NULL);
 	if (entry != NULL && rhythmdb_entry_get_entry_type (entry) == entry_type)
 		matches = TRUE;
-	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+	g_object_unref (entry_type);
 
 	return matches;
 }
