@@ -37,6 +37,8 @@
 
 #include <X11/Xatom.h>
 
+#include <pango/pango-bidi-type.h>
+
 #ifdef HAVE_NOTIFY
 #include <libnotify/notify.h>
 #endif
@@ -50,12 +52,9 @@
 #include "rb-stock-icons.h"
 #include "eel-gconf-extensions.h"
 #include "rb-builder-helpers.h"
+#include "gseal-gtk-compat.h"
 
-#if defined(USE_GTK_STATUS_ICON)
 #include "rb-tray-icon-gtk.h"
-#else
-#include "rb-tray-icon.h"
-#endif
 
 #define TRAY_ICON_DEFAULT_TOOLTIP _("Music Player")
 
@@ -281,6 +280,13 @@ rb_status_icon_plugin_button_press_event (RBStatusIconPlugin *plugin,
 		rb_tray_icon_menu_popup (plugin->priv->tray_icon, popup, 3);
 		g_object_unref (ui_manager);
 		break;
+
+	case 8:
+		rb_shell_player_do_previous (plugin->priv->shell_player, NULL);
+		break;
+	case 9:
+		rb_shell_player_do_next (plugin->priv->shell_player, NULL);
+		break;
 	}
 }
 
@@ -396,6 +402,7 @@ do_notify (RBStatusIconPlugin *plugin,
 					 G_CALLBACK (notification_closed_cb),
 					 plugin, 0);
 	} else {
+		notify_notification_clear_hints (plugin->priv->notification);
 		notify_notification_update (plugin->priv->notification, primary, secondary, icon_name);
 	}
 
@@ -417,6 +424,7 @@ do_notify (RBStatusIconPlugin *plugin,
 	notify_notification_set_timeout (plugin->priv->notification, timeout);
 
 	if (pixbuf != NULL) {
+		notify_notification_clear_hints (plugin->priv->notification);
 		notify_notification_set_icon_from_pixbuf (plugin->priv->notification, pixbuf);
 	}
 
@@ -633,6 +641,50 @@ rb_status_icon_plugin_set_tooltip (GtkWidget        *widget,
 
 /* information on current track */
 
+static void
+get_artist_album_templates (const char *artist,
+			    const char *album,
+			    const char **artist_template,
+			    const char **album_template)
+{
+	PangoDirection tag_dir;
+	PangoDirection template_dir;
+
+	/* Translators: by Artist */
+	*artist_template = _("by <i>%s</i>");
+	/* Translators: from Album */
+	*album_template = _("from <i>%s</i>");
+
+	/* find the direction (left-to-right or right-to-left) of the
+	 * track's tags and the localized templates
+	 */
+	if (artist != NULL && artist[0] != '\0') {
+		tag_dir = pango_find_base_dir (artist, -1);
+		template_dir = pango_find_base_dir (*artist_template, -1);
+	} else if (album != NULL && album[0] != '\0') {
+		tag_dir = pango_find_base_dir (album, -1);
+		template_dir = pango_find_base_dir (*album_template, -1);
+	} else {
+		return;
+	}
+
+	/* if the track's tags and the localized templates have a different
+	 * direction, switch to direction-neutral templates in order to improve
+	 * display.
+	 * text can have a neutral direction, this condition only applies when
+	 * both directions are defined and they are conflicting.
+	 * https://bugzilla.gnome.org/show_bug.cgi?id=609767
+	 */
+	if (((tag_dir == PANGO_DIRECTION_LTR) && (template_dir == PANGO_DIRECTION_RTL)) ||
+	    ((tag_dir == PANGO_DIRECTION_RTL) && (template_dir == PANGO_DIRECTION_LTR))) {
+		/* these strings should not be localized, they must be
+		 * locale-neutral and direction-neutral
+		 */
+		*artist_template = "<i>%s</i>";
+		*album_template = "/ <i>%s</i>";
+	}
+}
+
 static void
 update_current_playing_data (RBStatusIconPlugin *plugin, RhythmDBEntry *entry)
 {
@@ -643,6 +695,9 @@ update_current_playing_data (RBStatusIconPlugin *plugin, RhythmDBEntry *entry)
 	char *title = NULL;
 	GString *secondary;
 
+	const char *artist_template = NULL;
+	const char *album_template = NULL;
+
 	g_free (plugin->priv->current_title);
 	g_free (plugin->priv->current_album_and_artist);
 	plugin->priv->current_title = NULL;
@@ -665,12 +720,6 @@ update_current_playing_data (RBStatusIconPlugin *plugin, RhythmDBEntry *entry)
 		artist = markup_escape (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST));
 	}
 
-	if (artist != NULL && artist[0] != '\0') {
-		/* Translators: by Artist */
-		g_string_append_printf (secondary, _("by <i>%s</i>"), artist);
-	}
-	g_free (artist);
-
 	/* get album, preferring streaming song details */
 	value = rhythmdb_entry_request_extra_metadata (plugin->priv->db,
 						       entry,
@@ -683,12 +732,18 @@ update_current_playing_data (RBStatusIconPlugin *plugin, RhythmDBEntry *entry)
 		album = markup_escape (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM));
 	}
 
+	get_artist_album_templates (artist, album, &artist_template, &album_template);
+
+	if (artist != NULL && artist[0] != '\0') {
+		g_string_append_printf (secondary, artist_template, artist);
+	}
+	g_free (artist);
+
 	if (album != NULL && album[0] != '\0') {
 		if (secondary->len != 0)
 			g_string_append_c (secondary, ' ');
 
-		/* Translators: from Album */
-		g_string_append_printf (secondary, _("from <i>%s</i>"), album);
+		g_string_append_printf (secondary, album_template, album);
 	}
 	g_free (album);
 
@@ -808,6 +863,8 @@ db_art_metadata_cb (RhythmDB *db,
 		if (time < PLAYING_ENTRY_NOTIFY_TIME) {
 			notify_playing_entry (plugin, FALSE);
 		}
+	} else {
+		notify_playing_entry (plugin, FALSE);
 	}
 }
 
@@ -922,8 +979,8 @@ close_to_tray (RBStatusIconPlugin *plugin)
 	/* set the window's icon geometry to match the icon */
 	rb_tray_icon_get_geom (plugin->priv->tray_icon,
 			       &x, &y, &width, &height);
-	if (GTK_WIDGET_REALIZED (window))
-		set_icon_geometry (GTK_WIDGET (window)->window,
+	if (gtk_widget_get_realized (GTK_WIDGET (window)))
+		set_icon_geometry (gtk_widget_get_window (GTK_WIDGET (window)),
 				   x, y, width, height);
 
 	/* ask the tasklist not to show our window */
@@ -943,7 +1000,6 @@ visibility_changing_cb (RBShell *shell,
 			gboolean visible,
 			RBStatusIconPlugin *plugin)
 {
-
 	switch (plugin->priv->icon_mode) {
 	case ICON_NEVER:
 	case ICON_WITH_NOTIFY:
@@ -959,9 +1015,16 @@ visibility_changing_cb (RBShell *shell,
 	}
 
 	if (initial) {
-		/* restore visibility from gconf setting */
-		visible = eel_gconf_get_boolean (CONF_WINDOW_VISIBILITY);
-		rb_debug ("setting initial visibility %d from gconf", visible);
+		gboolean autostarted;
+		g_object_get (shell, "autostarted", &autostarted, NULL);
+		if (autostarted) {
+			/* restore visibility from gconf setting */
+			visible = eel_gconf_get_boolean (CONF_WINDOW_VISIBILITY) || eel_gconf_is_default (CONF_WINDOW_VISIBILITY);
+			rb_debug ("setting initial visibility %d from gconf", visible);
+		} else {
+			rb_debug ("ignoring stored visibility as we weren't autostarted");
+			visible = TRUE;
+		}
 		return visible;
 	}
 
@@ -969,8 +1032,13 @@ visibility_changing_cb (RBShell *shell,
 
 	if (visible) {
 		GtkWindow *window;
+		GdkWindow *gdkwindow;
 
 		g_object_get (shell, "window", &window, NULL);
+		gdkwindow = gtk_widget_get_window (GTK_WIDGET (window));
+		if (gdkwindow) {
+		    gdk_x11_window_move_to_current_desktop (gdkwindow);
+		}
 		gtk_window_set_skip_taskbar_hint (window, FALSE);
 		g_object_unref (window);
 	} else {
@@ -1019,80 +1087,6 @@ store_window_visibility (RBStatusIconPlugin *plugin)
 	}
 }
 
-#if !defined(USE_GTK_STATUS_ICON)
-
-/* EggTrayIcon helpers */
-
-#if 0
-static void
-tray_embedded_cb (GtkPlug *plug,
-		  gpointer data)
-{
-	/* FIXME - this doesn't work */
-	RBShell *shell = RB_SHELL (data);
-
-	rb_debug ("got embedded signal");
-
-	gdk_window_set_decorations (shell->priv->window->window,
-				    GDK_DECOR_ALL | GDK_DECOR_MINIMIZE | GDK_DECOR_MAXIMIZE);
-}
-#endif
-
-static gboolean
-tray_destroy_cb (GtkObject *object, RBStatusIconPlugin *plugin)
-{
-	if (plugin->priv->tray_icon) {
-		rb_debug ("caught destroy event for icon %p", object);
-		g_object_ref_sink (object);
-		plugin->priv->tray_icon = NULL;
-		rb_debug ("finished sinking tray");
-	}
-
-	rb_debug ("creating new icon");
-	plugin->priv->tray_icon = rb_tray_icon_new (plugin, plugin->priv->shell);
-	g_signal_connect_object (plugin->priv->tray_icon, "destroy", G_CALLBACK (tray_destroy_cb), plugin, 0);
-	/* g_signal_connect_object (plugin->priv->tray_icon, "embedded", G_CALLBACK (tray_embedded_cb), plugin, 0); */
-
-	rb_debug ("done creating new icon %p", plugin->priv->tray_icon);
-	return TRUE;
-}
-
-static void
-cleanup_status_icon (RBStatusIconPlugin *plugin)
-{
-	g_signal_handlers_disconnect_by_func (plugin->priv->tray_icon,
-					      G_CALLBACK (tray_destroy_cb),
-					      plugin);
-
-	gtk_widget_hide_all (GTK_WIDGET (plugin->priv->tray_icon));
-	gtk_widget_destroy (GTK_WIDGET (plugin->priv->tray_icon));
-}
-
-static void
-create_status_icon (RBStatusIconPlugin *plugin)
-{
-	tray_destroy_cb (NULL, plugin);
-}
-
-#else
-
-/* boring equivalents for GtkStatusIcon */
-
-static void
-create_status_icon (RBStatusIconPlugin *plugin)
-{
-	plugin->priv->tray_icon = rb_tray_icon_new (plugin, plugin->priv->shell_player);
-}
-
-static void
-cleanup_status_icon (RBStatusIconPlugin *plugin)
-{
-	g_object_unref (plugin->priv->tray_icon);
-}
-
-
-#endif
-
 /* preferences dialog and gconf stuff */
 
 static void
@@ -1334,7 +1328,7 @@ impl_activate (RBPlugin *bplugin,
 	plugin->priv->wheel_mode = eel_gconf_get_integer (CONF_MOUSE_WHEEL_MODE);
 
 	/* create status icon */
-	create_status_icon (plugin);
+	plugin->priv->tray_icon = rb_tray_icon_new (plugin, plugin->priv->shell_player);
 	update_status_icon_visibility (plugin, FALSE);
 
 	/* update everything in case we're already playing something */
@@ -1387,7 +1381,7 @@ impl_deactivate	(RBPlugin *bplugin,
 
 	/* remove icon */
 	if (plugin->priv->tray_icon != NULL) {
-		cleanup_status_icon (plugin);
+		g_object_unref (plugin->priv->tray_icon);
 		plugin->priv->tray_icon = NULL;
 	}
 
