@@ -7,7 +7,7 @@
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -34,7 +34,7 @@
 #include <glib/gi18n.h>
 #include <glib-object.h>
 #include <dbus/dbus-glib.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
+#include <gio/gio.h>
 
 #include "rb-debug.h"
 #include "rb-shell-binding.h"
@@ -50,11 +50,12 @@ static gboolean hide = FALSE;
 
 static gboolean next = FALSE;
 static gboolean previous = FALSE;
+static gint32 seek = 0;
 
 static gboolean notify = FALSE;
 
 static gboolean play = FALSE;
-static gboolean pause = FALSE;
+static gboolean do_pause = FALSE;
 static gboolean play_pause = FALSE;
 static gboolean stop = FALSE;
 
@@ -62,6 +63,9 @@ static gboolean enqueue = FALSE;
 
 static gboolean clear_queue = FALSE;
 
+static gchar *select_source = NULL;
+static gchar *activate_source = NULL;
+static gchar *play_source = NULL;
 static gchar *play_uri = NULL;
 static gboolean print_playing = FALSE;
 static gchar *print_playing_format = NULL;
@@ -72,6 +76,7 @@ static gboolean volume_down = FALSE;
 static gboolean print_volume = FALSE;
 static gboolean mute = FALSE;
 static gboolean unmute = FALSE;
+static gdouble set_rating = -1.0;
 
 static gchar **other_stuff = NULL;
 
@@ -86,11 +91,12 @@ static GOptionEntry args[] = {
 
 	{ "next", 0, 0, G_OPTION_ARG_NONE, &next, N_("Jump to next song"), NULL },
 	{ "previous", 0, 0, G_OPTION_ARG_NONE, &previous, N_("Jump to previous song"), NULL },
+	{ "seek", 0, 0, G_OPTION_ARG_INT64, &seek, N_("Seek in current track"), NULL },
 
 	{ "notify", 0, 0, G_OPTION_ARG_NONE, &notify, N_("Show notification of the playing song"), NULL },
 
 	{ "play", 0, 0, G_OPTION_ARG_NONE, &play, N_("Resume playback if currently paused"), NULL },
-	{ "pause", 0, 0, G_OPTION_ARG_NONE, &pause, N_("Pause playback if currently playing"), NULL },
+	{ "pause", 0, 0, G_OPTION_ARG_NONE, &do_pause, N_("Pause playback if currently playing"), NULL },
 	{ "play-pause", 0, 0, G_OPTION_ARG_NONE, &play_pause, N_("Toggle play/pause mode"), NULL },
 /*	{ "stop", 0, 0, G_OPTION_ARG_NONE, &stop, N_("Stop playback"), NULL }, */
 
@@ -100,21 +106,26 @@ static GOptionEntry args[] = {
 
 	{ "print-playing", 0, 0, G_OPTION_ARG_NONE, &print_playing, N_("Print the title and artist of the playing song"), NULL },
 	{ "print-playing-format", 0, 0, G_OPTION_ARG_STRING, &print_playing_format, N_("Print formatted details of the song"), NULL },
+	{ "select-source", 0, 0, G_OPTION_ARG_STRING, &select_source, N_("Select the source matching the specified URI"), N_("Source to select")},
+	{ "activate-source", 0, 0, G_OPTION_ARG_STRING, &activate_source, N_("Activate the source matching the specified URI"), N_("Source to activate")},
+	{ "play-source", 0, 0, G_OPTION_ARG_STRING, &play_source, N_("Play from the source matching the specified URI"), N_("Source to play from")},
 
-#if GLIB_CHECK_VERSION(2,12,0)
 	{ "set-volume", 0, 0, G_OPTION_ARG_DOUBLE, &set_volume, N_("Set the playback volume"), NULL },
-#endif
 	{ "volume-up", 0, 0, G_OPTION_ARG_NONE, &volume_up, N_("Increase the playback volume"), NULL },
 	{ "volume-down", 0, 0, G_OPTION_ARG_NONE, &volume_down, N_("Decrease the playback volume"), NULL },
 	{ "print-volume", 0, 0, G_OPTION_ARG_NONE, &print_volume, N_("Print the current playback volume"), NULL },
 	{ "mute", 0, 0, G_OPTION_ARG_NONE, &mute, N_("Mute playback"), NULL },
 	{ "unmute", 0, 0, G_OPTION_ARG_NONE, &unmute, N_("Unmute playback"), NULL },
+	{ "set-rating", 0, 0, G_OPTION_ARG_DOUBLE, &set_rating, N_("Set the rating of the current song"), NULL },
 
 	{ G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &other_stuff, NULL, NULL },
 
 	{ NULL }
 };
 
+static gboolean db_load_complete = FALSE;
+static gboolean scan_finished = FALSE;
+
 static gboolean
 annoy (GError **error)
 {
@@ -172,6 +183,8 @@ rb_make_duration_string (guint duration)
  * %tS -- track artist sortname (lowercase)
  * %td -- track duration
  * %te -- track elapsed time
+ * %tb -- track bitrate
+ * %st -- stream title
  */
 static char *
 parse_pattern (const char *pattern, GHashTable *properties, guint elapsed)
@@ -231,14 +244,16 @@ parse_pattern (const char *pattern, GHashTable *properties, guint elapsed)
 				if (value)
 					string = g_value_dup_string (value);
 				break;
-			/*case 's':
-				string = sanitize_path (album sort name);
+			case 's':
+				value = g_hash_table_lookup (properties, "mb-artistsortname");
+				if (value)
+					string = g_value_dup_string (value);
 				break;
 			case 'S':
-				char *t = g_utf8_strdown (album sort name)
-				string = sanitize_path (t);
-				g_free (t);
-				break;*/
+				value = g_hash_table_lookup (properties, "mb-artistsortname");
+				if (value)
+					string = g_utf8_strdown (g_value_get_string (value), -1);
+				break;
 			case 'y':
 				/* Release year */
 				value = g_hash_table_lookup (properties, "year");
@@ -298,14 +313,16 @@ parse_pattern (const char *pattern, GHashTable *properties, guint elapsed)
 				if (value)
 					string = g_value_dup_string (value);
 				break;
-			/*case 's':
-				string = sanitize_path (artist sort name);
+			case 's':
+				value = g_hash_table_lookup (properties, "mb-artistsortname");
+				if (value)
+					string = g_value_dup_string (value);
 				break;
 			case 'S':
-				char *t = g_utf8_strdown (artist sort name)
-				string = sanitize_path (t);
-				g_free (t);
-				break;*/
+				value = g_hash_table_lookup (properties, "mb-artistsortname");
+				if (value)
+					string = g_utf8_strdown (g_value_get_string (value), -1);
+				break;
 			case 'n':
 				/* Track number */
 				value = g_hash_table_lookup (properties, "track-number");
@@ -328,12 +345,36 @@ parse_pattern (const char *pattern, GHashTable *properties, guint elapsed)
 				/* Track elapsed time */
 				string = rb_make_duration_string (elapsed);
 				break;
+			case 'b':
+				/* Track bitrate */
+				value = g_hash_table_lookup (properties, "bitrate");
+				if (value)
+					string = g_strdup_printf ("%u", g_value_get_uint (value));
+				break;
+
 			default:
 				string = g_strdup_printf ("%%t%c", *p);
  			}
 
 			break;
 
+		case 's':
+			/*
+			 * Stream tag
+			 */
+			switch (*++p) {
+			case 't':
+				value = g_hash_table_lookup (properties, "rb:stream-song-title");
+				if (value)
+					string = g_value_dup_string (value);
+				break;
+			default:
+				string = g_strdup_printf ("%%s%c", *p);
+ 			}
+
+			break;
+
+
 		default:
 			string = g_strdup_printf ("%%%c", *p);
 		}
@@ -367,6 +408,8 @@ create_rb_shell_proxies (DBusGConnection *bus, DBusGProxy **shell_proxy, DBusGPr
 		*shell_proxy = NULL;
 		return ((*error)->code == DBUS_GERROR_NAME_HAS_NO_OWNER);
 	}
+	dbus_g_proxy_add_signal (sp, "removableMediaScanFinished", G_TYPE_INVALID);
+	dbus_g_proxy_add_signal (sp, "databaseLoadComplete", G_TYPE_INVALID);
 
 	rb_debug ("creating player proxy");
 	*player_proxy = dbus_g_proxy_new_from_proxy (sp,
@@ -383,31 +426,49 @@ create_rb_shell_proxies (DBusGConnection *bus, DBusGProxy **shell_proxy, DBusGPr
 	return TRUE;
 }
 
+static GHashTable *
+get_playing_song_info (DBusGProxy *shell_proxy, DBusGProxy *player_proxy, GError **error)
+{
+	char *playing_uri;
+	GHashTable *properties;
+
+	org_gnome_Rhythmbox_Player_get_playing_uri (player_proxy, &playing_uri, error);
+	if (*error != NULL) {
+		return NULL;
+	}
+
+	if (!playing_uri || playing_uri[0] == '\0') {
+		return NULL;
+	}
+
+	rb_debug ("playing song is %s", playing_uri);
+	org_gnome_Rhythmbox_Shell_get_song_properties (shell_proxy, playing_uri, &properties, error);
+	if (*error != NULL) {
+		return NULL;
+	}
+
+	return properties;
+}
+
 static void
 print_playing_song (DBusGProxy *shell_proxy, DBusGProxy *player_proxy, const char *format)
 {
-	char *playing_uri;
-	GError *error = NULL;
 	GHashTable *properties;
-	char *string;
 	guint elapsed = 0;
+	GError *error = NULL;
+	char *string;
 
-	org_gnome_Rhythmbox_Player_get_playing_uri (player_proxy, &playing_uri, &error);
-	if (annoy (&error))
+	properties = get_playing_song_info (shell_proxy, player_proxy, &error);
+	if (annoy (&error)) {
 		return;
-
-	org_gnome_Rhythmbox_Player_get_elapsed (player_proxy, &elapsed, &error);
-	annoy (&error);
-
-	if (!playing_uri || playing_uri[0] == '\0') {
+	}
+	if (properties == NULL) {
 		g_print ("%s\n", _("Not playing"));
 		return;
 	}
-	rb_debug ("playing song is %s", playing_uri);
-
-	org_gnome_Rhythmbox_Shell_get_song_properties (shell_proxy, playing_uri, &properties, &error);
-	if (annoy (&error))
-		return;
+	
+	org_gnome_Rhythmbox_Player_get_elapsed (player_proxy, &elapsed, &error);
+	annoy (&error);
 
 	string = parse_pattern (format, properties, elapsed);
 	g_print ("%s\n", string);
@@ -415,6 +476,91 @@ print_playing_song (DBusGProxy *shell_proxy, DBusGProxy *player_proxy, const cha
 	g_free (string);
 }
 
+static void
+print_playing_song_default (DBusGProxy *shell_proxy, DBusGProxy *player_proxy)
+{
+	GHashTable *properties;
+	char *string;
+	GError *error = NULL;
+
+	properties = get_playing_song_info (shell_proxy, player_proxy, &error);
+	if (annoy (&error)) {
+		return;
+	}
+	if (properties == NULL) {
+		g_print ("%s\n", _("Not playing"));
+		return;
+	}
+
+	if (g_hash_table_lookup (properties, "rb:stream-song-title") != NULL) {
+		string = parse_pattern ("%st (%tt)", properties, 0);
+	} else {
+		string = parse_pattern ("%ta - %tt", properties, 0);
+	}
+
+	g_print ("%s\n", string);
+	g_hash_table_destroy (properties);
+	g_free (string);
+}
+
+static void
+rate_song (DBusGProxy *shell_proxy, DBusGProxy *player_proxy, gdouble song_rating)
+{
+	GError *error = NULL;
+	char *playing_uri;
+	GValue *value = NULL;
+
+	org_gnome_Rhythmbox_Player_get_playing_uri (player_proxy, &playing_uri, &error);
+	if (annoy (&error)) {
+		return;
+	}
+
+	if (!playing_uri || playing_uri[0] == '\0') {
+		return;
+	}
+
+	rb_debug ("rating song %s: %f", playing_uri, song_rating);
+
+	value = g_new0 (GValue, 1);
+	g_value_init (value, G_TYPE_DOUBLE);
+	g_value_set_double (value, song_rating);
+	org_gnome_Rhythmbox_Shell_set_song_property (shell_proxy, playing_uri, "rating", value, &error);
+	annoy (&error);
+
+	g_free (value);
+	g_free (playing_uri);
+}
+
+static void
+maybe_unblock (GMainLoop *loop)
+{
+	if (db_load_complete && scan_finished) {
+		/* FIXME - we're not really ready to accept playback control just yet.
+		 * the library (at least) needs to populate itself with the database contents,
+		 * which takes some non-zero amount of time.  this hack makes things work
+		 * somewhat reliably.
+		 */
+		g_usleep (G_USEC_PER_SEC);
+		g_main_loop_quit (loop);
+	}
+}
+
+static void
+scan_finished_cb (DBusGProxy *proxy, GMainLoop *loop)
+{
+	rb_debug ("removable media scan finished");
+	scan_finished = TRUE;
+	maybe_unblock (loop);
+}
+
+static void
+db_load_complete_cb (DBusGProxy *proxy, GMainLoop *loop)
+{
+	rb_debug ("database load complete");
+	db_load_complete = TRUE;
+	maybe_unblock (loop);
+}
+
 int
 main (int argc, char **argv)
 {
@@ -470,6 +616,7 @@ main (int argc, char **argv)
 	}
 	if (shell_proxy == NULL) {
 		DBusGProxy *bus_proxy;
+		GMainLoop *loop;
 		guint start_service_reply;
 
 		if (no_start) {
@@ -499,15 +646,22 @@ main (int argc, char **argv)
 			exit (1);
 		}
 		g_clear_error (&error);
+
+		/* wait for it to load enough state to be useful */
+		rb_debug ("waiting for database load and removable media scan to finish");
+		loop = g_main_loop_new (NULL, FALSE);
+		dbus_g_proxy_connect_signal (shell_proxy, "removableMediaScanFinished", G_CALLBACK (scan_finished_cb), loop, NULL);
+		dbus_g_proxy_connect_signal (shell_proxy, "databaseLoadComplete", G_CALLBACK (db_load_complete_cb), loop, NULL);
+		g_main_loop_run (loop);
 	}
 
 	/* don't present if we're doing something else */
-	if (next || previous ||
+	if (next || previous || (seek != 0) ||
 	    clear_queue ||
 	    play_uri || other_stuff ||
-	    play || pause || play_pause || stop ||
+	    play || do_pause || play_pause || stop ||
 	    print_playing || print_playing_format || notify ||
-	    (set_volume > -0.01) || volume_up || volume_down || print_volume || mute || unmute)
+	    (set_volume > -0.01) || volume_up || volume_down || print_volume || mute || unmute || (set_rating > -0.01))
 		no_present = TRUE;
 
 	/* 2. present or hide */
@@ -540,6 +694,13 @@ main (int argc, char **argv)
 		annoy (&error);
 	}
 
+	/* seek in track */
+	if (seek != 0) {
+		rb_debug ("seek");
+		org_gnome_Rhythmbox_Player_seek (player_proxy, seek, &error);
+		annoy (&error);
+	}
+
 	/* 4. add/enqueue */
 	if (clear_queue) {
 		org_gnome_Rhythmbox_Shell_clear_queue (shell_proxy, &error);
@@ -548,42 +709,64 @@ main (int argc, char **argv)
 	if (other_stuff) {
 		int i;
 		for (i = 0; other_stuff[i] != NULL; i++) {
-			char *uri = gnome_vfs_make_uri_from_shell_arg (other_stuff[i]);
-			if (uri == NULL) {
+			GFile *file;
+			char *fileuri;
+
+			file = g_file_new_for_commandline_arg (other_stuff[i]);
+			fileuri = g_file_get_uri (file);
+			if (fileuri == NULL) {
 				g_warning ("couldn't convert \"%s\" to a URI", other_stuff[i]);
 				continue;
 			}
 
 			if (enqueue) {
-				rb_debug ("enqueueing %s", uri);
-				org_gnome_Rhythmbox_Shell_add_to_queue (shell_proxy, uri, &error);
+				rb_debug ("enqueueing %s", fileuri);
+				org_gnome_Rhythmbox_Shell_add_to_queue (shell_proxy, fileuri, &error);
 			} else {
-				rb_debug ("importing %s", uri);
-				org_gnome_Rhythmbox_Shell_load_ur_i (shell_proxy, uri, FALSE, &error);
+				rb_debug ("importing %s", fileuri);
+				org_gnome_Rhythmbox_Shell_load_ur_i (shell_proxy, fileuri, FALSE, &error);
 			}
 			annoy (&error);
-			g_free (uri);
+			g_free (fileuri);
+			g_object_unref (file);
 		}
 	}
 
+	/* select/activate/play source */
+	if (select_source) {
+		org_gnome_Rhythmbox_Shell_activate_source (shell_proxy, select_source, 0, &error);
+		annoy (&error);
+	} else if (activate_source) {
+		org_gnome_Rhythmbox_Shell_activate_source (shell_proxy, activate_source, 1, &error);
+		annoy (&error);
+	} else if (play_source) {
+		org_gnome_Rhythmbox_Shell_activate_source (shell_proxy, play_source, 2, &error);
+		annoy (&error);
+	}
+
 	/* play uri */
 	if (play_uri) {
-		char *uri = gnome_vfs_make_uri_from_shell_arg (play_uri);
-		if (uri == NULL) {
+		GFile *file;
+		char *fileuri;
+
+		file = g_file_new_for_commandline_arg (play_uri);
+		fileuri = g_file_get_uri (file);
+		if (fileuri == NULL) {
 			g_warning ("couldn't convert \"%s\" to a URI", play_uri);
 		} else {
-			rb_debug ("loading and playing %s", uri);
-			org_gnome_Rhythmbox_Shell_load_ur_i (shell_proxy, uri, TRUE, &error);
+			rb_debug ("loading and playing %s", fileuri);
+			org_gnome_Rhythmbox_Shell_load_ur_i (shell_proxy, fileuri, TRUE, &error);
 			annoy (&error);
 		}
-		g_free (uri);
+		g_free (fileuri);
+		g_object_unref (file);
 	}
 
 	/* 5. play/pause/stop */
 	org_gnome_Rhythmbox_Player_get_playing (player_proxy, &is_playing, &error);
 	if (!annoy (&error)) {
 		rb_debug ("playback state: %d", is_playing);
-		if (play || pause || play_pause) {
+		if (play || do_pause || play_pause) {
 			if (is_playing != play || play_pause) {
 				rb_debug ("calling playPause to change playback state");
 				org_gnome_Rhythmbox_Player_play_pause (player_proxy, FALSE, &error);
@@ -597,13 +780,10 @@ main (int argc, char **argv)
 	}
 
 	/* 6. get/set volume, mute/unmute */
-#if GLIB_CHECK_VERSION(2,12,0)
 	if (set_volume > -0.01) {
 		org_gnome_Rhythmbox_Player_set_volume (player_proxy, set_volume, &error);
 		annoy (&error);
-	} else
-#endif
-	if (volume_up || volume_down) {
+	} else if (volume_up || volume_down) {
 		org_gnome_Rhythmbox_Player_set_volume_relative (player_proxy, volume_up ? 0.1 : -0.1, &error);
 		annoy (&error);
 	} else if (unmute || mute) {
@@ -629,7 +809,7 @@ main (int argc, char **argv)
 	if (print_playing_format) {
 		print_playing_song (shell_proxy, player_proxy, print_playing_format);
 	} else if (print_playing) {
-		print_playing_song (shell_proxy, player_proxy, "%ta - %tt");
+		print_playing_song_default (shell_proxy, player_proxy);
 	}
 
 	/* 8. display notification about playing song */
@@ -639,6 +819,13 @@ main (int argc, char **argv)
 		annoy (&error);
 	}
 
+	/* 9. set song rate */
+	if (set_rating >= 0.0 && set_rating <= 5.0) {
+		rb_debug ("rate song");
+
+		rate_song (shell_proxy, player_proxy, set_rating);
+	}
+
 	g_object_unref (shell_proxy);
 	g_object_unref (player_proxy);
 	g_option_context_free (context);
