@@ -52,6 +52,8 @@
 #include <glib/gi18n.h>
 #include <glib-object.h>
 
+#include "rb-track-transfer-batch.h"
+#include "rb-track-transfer-queue.h"
 #include <profiles/gnome-media-profiles.h>
 #include <profiles/audio-profile-choose.h>
 
@@ -63,8 +65,9 @@
 #include "rb-util.h"
 #include "eel-gconf-extensions.h"
 #include "rb-library-source.h"
-#include "rb-removable-media-manager.h"
 #include "rb-auto-playlist-source.h"
+#include "rb-encoder.h"
+#include "rb-missing-plugins.h"
 
 static void rb_library_source_class_init (RBLibrarySourceClass *klass);
 static void rb_library_source_init (RBLibrarySource *source);
@@ -79,9 +82,15 @@ static char *impl_get_browser_key (RBSource *source);
 static char *impl_get_paned_key (RBBrowserSource *source);
 static gboolean impl_receive_drag (RBSource *source, GtkSelectionData *data);
 static gboolean impl_can_paste (RBSource *asource);
-static void impl_paste (RBSource *source, GList *entries);
+static RBTrackTransferBatch *impl_paste (RBSource *source, GList *entries);
 static guint impl_want_uri (RBSource *source, const char *uri);
-static gboolean impl_add_uri (RBSource *source, const char *uri, const char *title, const char *genre);
+static gboolean impl_add_uri (RBSource *source,
+			      const char *uri,
+			      const char *title,
+			      const char *genre,
+			      RBSourceAddCallback callback,
+			      gpointer data,
+			      GDestroyNotify destroy_data);
 static void impl_get_status (RBSource *source, char **text, char **progress_text, float *progress);
 
 static void rb_library_source_ui_prefs_sync (RBLibrarySource *source);
@@ -390,9 +399,6 @@ rb_library_source_new (RBShell *shell)
 	RBSource *source;
 	GdkPixbuf *icon;
 	gint size;
-	RhythmDBEntryType entry_type;
-
-	entry_type = RHYTHMDB_ENTRY_TYPE_SONG;
 
 	gtk_icon_size_lookup (RB_SOURCE_ICON_SIZE, &size, NULL);
 	icon = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
@@ -401,7 +407,7 @@ rb_library_source_new (RBShell *shell)
 					 0, NULL);
 	source = RB_SOURCE (g_object_new (RB_TYPE_LIBRARY_SOURCE,
 					  "name", _("Music"),
-					  "entry-type", entry_type,
+					  "entry-type", RHYTHMDB_ENTRY_TYPE_SONG,
 					  "source-group", RB_SOURCE_GROUP_LIBRARY,
 					  "sorting-key", CONF_STATE_LIBRARY_SORTING,
 					  "shell", shell,
@@ -412,7 +418,7 @@ rb_library_source_new (RBShell *shell)
 		g_object_unref (icon);
 	}
 
-	rb_shell_register_entry_type_for_source (shell, source, entry_type);
+	rb_shell_register_entry_type_for_source (shell, source, RHYTHMDB_ENTRY_TYPE_SONG);
 
 	return source;
 }
@@ -781,7 +787,7 @@ rb_library_source_format_changed_cb (GtkWidget *widget, RBLibrarySource *source)
 	layout_example_label_update (source);
 }
 
-/**
+/*
  * Perform magic on a path to make it safe.
  *
  * This will always replace '/' with '-', and optionally make the file name
@@ -837,7 +843,7 @@ sanitize_pattern (const char *pat)
  * %aA -- album artist (lowercase)
  * %as -- album artist sortname
  * %aS -- album artist sortname (lowercase)
- * %aY -- album release year
+ * %ay -- album release year
  * %an -- album disc number
  * %aN -- album disc number, zero padded
  * %ag -- album genre
@@ -859,28 +865,27 @@ filepath_parse_pattern (RhythmDB *db,
 	const char *p;
 	char *temp;
 	GString *s;
-	GValue *value;
 	RBRefString *albumartist;
+	RBRefString *albumartist_sort;
 
 	if (pattern == NULL || pattern[0] == 0)
 		return g_strdup (" ");
 
-	/* request album artist (this is sort of temporary) */
-	value = rhythmdb_entry_request_extra_metadata (db, entry, RHYTHMDB_PROP_ALBUM_ARTIST);
-	if (value != NULL) {
-		albumartist = rb_refstring_new (g_value_get_string (value));
-		g_value_unset (value);
-		g_free (value);
-	} else {
+	/* figure out album artist - use the plain artist field if not specified */
+	albumartist = rhythmdb_entry_get_refstring (entry, RHYTHMDB_PROP_ALBUM_ARTIST);
+	if (albumartist == NULL || g_strcmp0 (rb_refstring_get (albumartist), "") == 0) {
 		albumartist = rhythmdb_entry_get_refstring (entry, RHYTHMDB_PROP_ARTIST);
 	}
+	albumartist_sort = rhythmdb_entry_get_refstring (entry, RHYTHMDB_PROP_ALBUM_ARTIST_SORTNAME);
+	if (albumartist_sort == NULL || g_strcmp0 (rb_refstring_get (albumartist_sort), "") == 0) {
+		albumartist_sort = rhythmdb_entry_get_refstring (entry, RHYTHMDB_PROP_ARTIST_SORTNAME);
+	}
 
 	s = g_string_new (NULL);
 
 	p = pattern;
 	while (*p) {
 		char *string = NULL;
-		char *t;
 
 		/* If not a % marker, copy and continue */
 		if (*p != '%') {
@@ -915,12 +920,10 @@ filepath_parse_pattern (RhythmDB *db,
 				string = sanitize_path (rb_refstring_get_folded (albumartist));
 				break;
 			case 's':
-				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_SORTNAME));
+				string = sanitize_path (rb_refstring_get (albumartist_sort));
 				break;
 			case 'S':
-				t = g_utf8_strdown (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_SORTNAME), -1);
-				string = sanitize_path (t);
-				g_free (t);
+				string = sanitize_path (rb_refstring_get_folded (albumartist_sort));
 				break;
 			case 'y':
 				string = g_strdup_printf ("%u", (guint)rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_YEAR));
@@ -964,9 +967,7 @@ filepath_parse_pattern (RhythmDB *db,
 				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_SORTNAME));
 				break;
 			case 'S':
-				t = g_utf8_strdown (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_SORTNAME), -1);
-				string = sanitize_path (t);
-				g_free (t);
+				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_SORTNAME_FOLDED));
 				break;
 			case 'n':
 				/* Track number */
@@ -1010,7 +1011,7 @@ layout_example_label_update (RBLibrarySource *source)
 	char *format;
 	char *tmp;
 	GMAudioProfile *profile;
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 	RhythmDBEntry *sample_entry;
 
   	profile = gm_audio_profile_choose_get_active (source->priv->preferred_format_menu);
@@ -1031,7 +1032,7 @@ layout_example_label_update (RBLibrarySource *source)
 
 	g_object_get (source, "entry-type", &entry_type, NULL);
 	sample_entry = rhythmdb_entry_example_new (source->priv->db, entry_type, NULL);
-	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+	g_object_unref (entry_type);
 
 	file_value = filepath_parse_pattern (source->priv->db, file_pattern, sample_entry);
 	path_value = filepath_parse_pattern (source->priv->db, path_pattern, sample_entry);
@@ -1130,7 +1131,7 @@ rb_library_source_layout_filename_changed (GConfClient *client,
 	layout_example_label_update (source);
 }
 
-/**
+/*
  * Build the absolute filename for the specified track.
  *
  * The base path is the extern variable 'base_path', the format to use
@@ -1140,7 +1141,7 @@ rb_library_source_layout_filename_changed (GConfClient *client,
  * Stolen from Sound-Juicer
  */
 static char*
-build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
+build_filename (RBLibrarySource *source, RhythmDBEntry *entry, const char *extension)
 {
 	GFile *library_location;
 	GFile *dir;
@@ -1149,19 +1150,16 @@ build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 	char *realpath;
 	char *filename;
 	char *string = NULL;
-	char *extension = NULL;
 	char *tmp;
 	GSList *list;
 	char *layout_path;
 	char *layout_filename;
-	char *preferred_format;
 
 	list = eel_gconf_get_string_list (CONF_LIBRARY_LOCATION);
 	layout_path = eel_gconf_get_string (CONF_LIBRARY_LAYOUT_PATH);
 	layout_filename = eel_gconf_get_string (CONF_LIBRARY_LAYOUT_FILENAME);
-	preferred_format = eel_gconf_get_string (CONF_LIBRARY_PREFERRED_FORMAT);
 
-	if (list == NULL || layout_path == NULL || layout_filename == NULL || preferred_format == NULL) {
+	if (list == NULL || layout_path == NULL || layout_filename == NULL) {
 		/* emit warning */
 		rb_debug ("Could not retrieve settings from GConf");
 		goto out;
@@ -1178,33 +1176,6 @@ build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 	g_object_unref (library_location);
 	g_free (realpath);
 
-	if (g_str_has_prefix (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MIMETYPE), "audio/x-raw")) {
-		GMAudioProfile *profile;
-		profile = gm_audio_profile_lookup (preferred_format);
-		if (profile)
-			extension = g_strdup (gm_audio_profile_get_extension (profile));
-	}
-
-	if (extension == NULL) {
-		const char *uri;
-		const char *loc;
-		char *tmp;
-
-		/* use the old extension. strip anything after a '?' for http/daap/etc */
-		uri = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
-		loc = g_utf8_strrchr (uri, -1, '.');
-		if (loc == NULL)
-			loc = g_utf8_strrchr (uri, -1, '/');
-		if (loc == NULL)
-			loc = uri;
-
-		extension = g_strdup (loc + 1);
-
-		tmp = g_utf8_strchr (extension, -1, '?');
-		if (tmp)
-			*tmp = '\0';
-	}
-
 	realfile = filepath_parse_pattern (source->priv->db, layout_filename, entry);
 	if (extension) {
 		filename = g_strdup_printf ("%s.%s", realfile, extension);
@@ -1215,7 +1186,6 @@ build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 
 	dest = g_file_resolve_relative_path (dir, filename);
 	g_object_unref (dir);
-	g_free (extension);
 	g_free (filename);
 
 	string = g_file_get_uri (dest);
@@ -1224,7 +1194,6 @@ build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 	rb_slist_deep_free (list);
 	g_free (layout_path);
 	g_free (layout_filename);
-	g_free (preferred_format);
 
 	return string;
 }
@@ -1254,33 +1223,80 @@ impl_can_paste (RBSource *asource)
 	return can_paste;
 }
 
-static void
-completed_cb (RhythmDBEntry *entry, const char *dest, guint64 dest_size, GError *error, RBLibrarySource *source)
+static char *
+get_dest_uri_cb (RBTrackTransferBatch *batch,
+		 RhythmDBEntry *entry,
+		 const char *mediatype,
+		 const char *extension,
+		 RBLibrarySource *source)
 {
-	if (error == NULL) {
-		rhythmdb_add_uri (source->priv->db, dest);
-	} else {
-		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
+	char *dest;
+	char *sane_dest;
+
+	dest = build_filename (source, entry, extension);
+	if (dest == NULL) {
+		rb_debug ("could not create destination path for entry");
+		return NULL;
+	}
+
+	sane_dest = rb_sanitize_uri_for_filesystem (dest);
+	g_free (dest);
+	rb_debug ("destination URI for %s is %s",
+		  rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION),
+		  sane_dest);
+	return sane_dest;
+}
+
+static void
+track_done_cb (RBTrackTransferBatch *batch,
+	       RhythmDBEntry *entry,
+	       const char *dest,
+	       guint64 dest_size,
+	       const char *dest_mediatype,
+	       GError *error,
+	       RBLibrarySource *source)
+{
+	if (error != NULL) {
+		/* probably want to cancel the batch on some errors:
+		 * - out of disk space / read only
+		 * - source has vanished (hmm, how would we know?)
+		 *
+		 * and we probably want to do something intelligent about some other errors:
+		 * - encoder pipeline errors?  hmm.
+		 */
+		if (g_error_matches (error, RB_ENCODER_ERROR, RB_ENCODER_ERROR_OUT_OF_SPACE) ||
+		    g_error_matches (error, RB_ENCODER_ERROR, RB_ENCODER_ERROR_DEST_READ_ONLY)) {
+			rb_debug ("fatal transfer error: %s", error->message);
+			rb_track_transfer_batch_cancel (batch);
+			rb_error_dialog (NULL, _("Error transferring track"), "%s", error->message);
+		} else if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
 			rb_debug ("not displaying 'file exists' error for %s", dest);
 		} else {
 			rb_error_dialog (NULL, _("Error transferring track"), "%s", error->message);
 		}
+	} else {
+		/* could probably do something smarter here to avoid
+		 * re-reading tags etc.
+		 */
+		rhythmdb_add_uri (source->priv->db, dest);
 	}
 }
 
-static void
+static RBTrackTransferBatch *
 impl_paste (RBSource *asource, GList *entries)
 {
 	RBLibrarySource *source = RB_LIBRARY_SOURCE (asource);
-	RBRemovableMediaManager *rm_mgr;
+	RBTrackTransferQueue *xferq;
 	GList *l;
 	GSList *sl;
 	RBShell *shell;
-	RhythmDBEntryType source_entry_type;
+	RhythmDBEntryType *source_entry_type;
+	RBTrackTransferBatch *batch;
+	gboolean start_batch = FALSE;
 
 	if (impl_can_paste (asource) == FALSE) {
 		g_warning ("RBLibrarySource impl_paste called when gconf keys unset");
-		return;
+		return NULL;
 	}
 
 	sl = eel_gconf_get_string_list (CONF_LIBRARY_LOCATION);
@@ -1289,15 +1305,17 @@ impl_paste (RBSource *asource, GList *entries)
 		      "shell", &shell,
 		      "entry-type", &source_entry_type,
 		      NULL);
-	g_object_get (shell, "removable-media-manager", &rm_mgr, NULL);
+	g_object_get (shell, "track-transfer-queue", &xferq, NULL);
 	g_object_unref (shell);
 
+	batch = rb_track_transfer_batch_new (NULL, NULL, NULL, G_OBJECT (source));
+	g_signal_connect_object (batch, "get-dest-uri", G_CALLBACK (get_dest_uri_cb), source, 0);
+	g_signal_connect_object (batch, "track-done", G_CALLBACK (track_done_cb), source, 0);
+
 	for (l = entries; l != NULL; l = g_list_next (l)) {
 		RhythmDBEntry *entry = (RhythmDBEntry *)l->data;
-		RhythmDBEntryType entry_type;
+		RhythmDBEntryType *entry_type;
 		RBSource *source_source;
-		char *dest;
-		char *sane_dest;
 
 		rb_debug ("pasting entry %s", rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
 
@@ -1314,22 +1332,20 @@ impl_paste (RBSource *asource, GList *entries)
 			continue;
 		}
 
-		dest = build_filename (source, entry);
-		if (dest == NULL) {
-			rb_debug ("could not create destination path for entry");
-			continue;
-		}
-
-		sane_dest = rb_sanitize_uri_for_filesystem (dest);
-		g_free (dest);
-
-		rb_removable_media_manager_queue_transfer (rm_mgr, entry,
-							  sane_dest, NULL,
-							  (RBTransferCompleteCallback)completed_cb, source);
+		rb_track_transfer_batch_add (batch, entry);
+		start_batch = TRUE;
 	}
-	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, source_entry_type);
+	g_object_unref (source_entry_type);
 
-	g_object_unref (rm_mgr);
+	if (start_batch) {
+		rb_track_transfer_queue_start_batch (xferq, batch);
+	} else {
+		g_object_unref (batch);
+		batch = NULL;
+	}
+
+	g_object_unref (xferq);
+	return batch;
 }
 
 static guint
@@ -1384,11 +1400,13 @@ maybe_create_import_job (RBLibrarySource *source)
 {
 	RhythmDBImportJob *job;
 	if (source->priv->import_jobs == NULL || source->priv->start_import_job_id == 0) {
+
 		rb_debug ("creating new import job");
 		job = rhythmdb_import_job_new (source->priv->db,
 					       RHYTHMDB_ENTRY_TYPE_SONG,
 					       RHYTHMDB_ENTRY_TYPE_IGNORE,
 					       RHYTHMDB_ENTRY_TYPE_IMPORT_ERROR);
+
 		g_signal_connect_object (job,
 					 "status-changed",
 					 G_CALLBACK (import_job_status_changed_cb),
@@ -1412,8 +1430,39 @@ maybe_create_import_job (RBLibrarySource *source)
 	return job;
 }
 
+struct ImportJobCallbackData {
+	char *uri;
+	RBSource *source;
+	RBSourceAddCallback callback;
+	gpointer data;
+	GDestroyNotify destroy_data;
+};
+
+static void
+import_job_callback_destroy (struct ImportJobCallbackData *data)
+{
+	if (data->destroy_data != NULL) {
+		data->destroy_data (data->data);
+	}
+	g_object_unref (data->source);
+	g_free (data->uri);
+	g_free (data);
+}
+
+static void
+import_job_callback_cb (RhythmDBImportJob *job, int total, struct ImportJobCallbackData *data)
+{
+	data->callback (data->source, data->uri, data->data);
+}
+
 static gboolean
-impl_add_uri (RBSource *asource, const char *uri, const char *title, const char *genre)
+impl_add_uri (RBSource *asource,
+	      const char *uri,
+	      const char *title,
+	      const char *genre,
+	      RBSourceAddCallback callback,
+	      gpointer data,
+	      GDestroyNotify destroy_data)
 {
 	RBLibrarySource *source = RB_LIBRARY_SOURCE (asource);
 	RhythmDBImportJob *job;
@@ -1422,6 +1471,18 @@ impl_add_uri (RBSource *asource, const char *uri, const char *title, const char
 
 	rb_debug ("adding uri %s to library", uri);
 	rhythmdb_import_job_add_uri (job, uri);
+
+	if (callback != NULL) {
+		struct ImportJobCallbackData *cbdata;
+
+		cbdata = g_new0 (struct ImportJobCallbackData, 1);
+		cbdata->uri = g_strdup (uri);
+		cbdata->source = g_object_ref (source);
+		cbdata->callback = callback;
+		cbdata->data = data;
+		cbdata->destroy_data = destroy_data;
+		g_signal_connect_data (job, "complete", G_CALLBACK (import_job_callback_cb), cbdata, (GClosureNotify) import_job_callback_destroy, 0);
+	}
 	return TRUE;
 }
 
@@ -1433,7 +1494,7 @@ rb_library_source_add_child_source (const char *path, RBLibrarySource *library_s
 	RBShell *shell;
 	char *name;
 	GdkPixbuf *icon;
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 	char *sort_column;
 	int sort_order;
 	GFile *file;
@@ -1470,7 +1531,7 @@ rb_library_source_add_child_source (const char *path, RBLibrarySource *library_s
 	rb_shell_append_source (shell, source, RB_SOURCE (library_source));
 	library_source->priv->child_sources = g_list_prepend (library_source->priv->child_sources, source);
 
-	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+	g_object_unref (entry_type);
 	g_object_unref (shell);
 	g_free (name);
 }
