@@ -1,7 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  arch-tag: Implementation of playlist source object
- *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
  *  Copyright (C) 2003 Colin Walters <walters@gnome.org>
  *
@@ -74,9 +72,7 @@
 
 static void rb_playlist_source_class_init (RBPlaylistSourceClass *klass);
 static void rb_playlist_source_init (RBPlaylistSource *source);
-static GObject *rb_playlist_source_constructor (GType type,
-						guint n_construct_properties,
-						GObjectConstructParam *construct_properties);
+static void rb_playlist_source_constructed (GObject *object);
 static void rb_playlist_source_dispose (GObject *object);
 static void rb_playlist_source_finalize (GObject *object);
 static void rb_playlist_source_set_property (GObject *object,
@@ -122,13 +118,25 @@ static void rb_playlist_source_songs_sort_order_changed_cb (RBEntryView *view,
 						RBStaticPlaylistSource *source);
 static char *rb_playlist_source_make_sorting_key (RBPlaylistSource *source);
 
+static void remove_from_playlist_cmd (GtkAction *action, RBSource *source);
+static char *impl_get_delete_action (RBSource *source);
+
 #define CONF_STATE_SORTING_PREFIX CONF_PREFIX "/state/sorting/"
 #define PLAYLIST_SOURCE_SONGS_POPUP_PATH "/PlaylistViewPopup"
 #define PLAYLIST_SOURCE_POPUP_PATH "/PlaylistSourcePopup"
 
+static GtkActionEntry rb_playlist_source_actions [] =
+{
+	{ "RemoveFromPlaylist", GTK_STOCK_REMOVE, N_("Remove From Playlist"), NULL,
+	  N_("Remove each selected song from the playlist"),
+	  G_CALLBACK (remove_from_playlist_cmd) },
+};
+
+
 struct RBPlaylistSourcePrivate
 {
 	RhythmDB *db;
+	GtkActionGroup *action_group;
 
 	GHashTable *entries;
 
@@ -167,7 +175,7 @@ rb_playlist_source_class_init (RBPlaylistSourceClass *klass)
 
 	object_class->dispose = rb_playlist_source_dispose;
 	object_class->finalize = rb_playlist_source_finalize;
-	object_class->constructor = rb_playlist_source_constructor;
+	object_class->constructed = rb_playlist_source_constructed;
 
 	object_class->set_property = rb_playlist_source_set_property;
 	object_class->get_property = rb_playlist_source_get_property;
@@ -181,8 +189,8 @@ rb_playlist_source_class_init (RBPlaylistSourceClass *klass)
 	source_class->impl_can_add_to_queue = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_song_properties = impl_song_properties;
-	source_class->impl_can_pause = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_show_popup = impl_show_popup;
+	source_class->impl_get_delete_action = impl_get_delete_action;
 
 	klass->impl_show_entry_view_popup = default_show_entry_view_popup;
 	klass->impl_mark_dirty = default_mark_dirty;
@@ -271,29 +279,34 @@ rb_playlist_source_set_db (RBPlaylistSource *source,
 
 }
 
-static GObject *
-rb_playlist_source_constructor (GType type,
-				guint n_construct_properties,
-				GObjectConstructParam *construct_properties)
+static void
+rb_playlist_source_constructed (GObject *object)
 {
 	GObject *shell_player;
 	RBPlaylistSource *source;
-	RBPlaylistSourceClass *klass;
 	RBShell *shell;
 	RhythmDB *db;
 	RhythmDBQueryModel *query_model;
 	char *sorting_key;
 
-	klass = RB_PLAYLIST_SOURCE_CLASS (g_type_class_peek (RB_TYPE_PLAYLIST_SOURCE));
-
-	source = RB_PLAYLIST_SOURCE (G_OBJECT_CLASS (rb_playlist_source_parent_class)->
-			constructor (type, n_construct_properties, construct_properties));
+	RB_CHAIN_GOBJECT_METHOD (rb_playlist_source_parent_class, constructed, object);
+	source = RB_PLAYLIST_SOURCE (object);
 
 	g_object_get (source, "shell", &shell, NULL);
 	g_object_get (shell, "db", &db, NULL);
 	shell_player = rb_shell_get_player (shell);
 	rb_playlist_source_set_db (source, db);
 	g_object_unref (db);
+
+	source->priv->action_group = _rb_source_register_action_group (RB_SOURCE (source),
+								       "PlaylistActions",
+								       NULL, 0,
+								       shell);
+	_rb_action_group_add_source_actions (source->priv->action_group,
+					     G_OBJECT (shell),
+					     rb_playlist_source_actions,
+					     G_N_ELEMENTS (rb_playlist_source_actions));
+
 	g_object_unref (shell);
 
 	source->priv->entries = g_hash_table_new_full (rb_refstring_hash, rb_refstring_equal,
@@ -356,9 +369,11 @@ rb_playlist_source_constructor (GType type,
  	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_QUALITY, FALSE);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_RATING, FALSE);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_PLAY_COUNT, FALSE);
+	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_COMMENT, FALSE);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_LOCATION, FALSE);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_LAST_PLAYED, FALSE);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_FIRST_SEEN, FALSE);
+	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_BPM, FALSE);
 	rb_entry_view_set_columns_clickable (source->priv->songs, FALSE);
 
 	rb_playlist_source_setup_entry_view (source, source->priv->songs);
@@ -366,8 +381,6 @@ rb_playlist_source_constructor (GType type,
 	gtk_container_add (GTK_CONTAINER (source), GTK_WIDGET (source->priv->songs));
 
 	gtk_widget_show_all (GTK_WIDGET (source));
-
-	return G_OBJECT (source);
 }
 
 static void
@@ -555,6 +568,52 @@ rb_playlist_source_drop_cb (GtkWidget *widget,
 	gtk_drag_finish (context, TRUE, FALSE, time);
 }
 
+#if TOTEM_PL_PARSER_CHECK_VERSION(2,29,1)
+
+static void
+set_field_from_property (TotemPlPlaylist *playlist,
+			 TotemPlPlaylistIter *iter,
+			 RhythmDBEntry *entry,
+			 RhythmDBPropType property,
+			 const char *field)
+{
+	const char *value;
+
+	value = rhythmdb_entry_get_string (entry, property);
+	if (value != NULL) {
+		totem_pl_playlist_set (playlist, iter, field, value, NULL);
+	}
+}
+
+static gboolean
+playlist_iter_foreach (GtkTreeModel *model,
+		       GtkTreePath *path,
+		       GtkTreeIter *iter,
+		       TotemPlPlaylist *playlist)
+{
+	TotemPlPlaylistIter pl_iter;
+	RhythmDBEntry *entry;
+
+	gtk_tree_model_get (model, iter, 0, &entry, -1);
+	if (entry == NULL) {
+		return FALSE;
+	}
+
+	totem_pl_playlist_append (playlist, &pl_iter);
+	set_field_from_property (playlist, &pl_iter, entry, RHYTHMDB_PROP_LOCATION, TOTEM_PL_PARSER_FIELD_URI);
+	set_field_from_property (playlist, &pl_iter, entry, RHYTHMDB_PROP_ARTIST, TOTEM_PL_PARSER_FIELD_AUTHOR);
+	set_field_from_property (playlist, &pl_iter, entry, RHYTHMDB_PROP_GENRE, TOTEM_PL_PARSER_FIELD_GENRE);
+	set_field_from_property (playlist, &pl_iter, entry, RHYTHMDB_PROP_ALBUM, TOTEM_PL_PARSER_FIELD_ALBUM);
+	set_field_from_property (playlist, &pl_iter, entry, RHYTHMDB_PROP_TITLE, TOTEM_PL_PARSER_FIELD_TITLE);
+
+	/* could possibly set duration, file size.. ? */
+
+	return FALSE;
+}
+
+
+#else
+
 static void
 playlist_iter_func (GtkTreeModel *model,
 		    GtkTreeIter *iter,
@@ -574,7 +633,7 @@ playlist_iter_func (GtkTreeModel *model,
 		*title = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_TITLE);
 	}
 	if (custom_title != NULL) {
-		*custom_title = FALSE;
+		*custom_title = TRUE;
 	}
 
 	if (entry != NULL) {
@@ -582,6 +641,8 @@ playlist_iter_func (GtkTreeModel *model,
 	}
 }
 
+#endif
+
 /**
  * rb_playlist_source_save_playlist:
  * @source: a #RBPlaylistSource
@@ -596,15 +657,19 @@ rb_playlist_source_save_playlist (RBPlaylistSource *source,
 				  const char *uri,
 				  RBPlaylistExportType export_type)
 {
-	TotemPlParser *playlist;
+	TotemPlParser *pl;
 	GError *error = NULL;
 	char *name;
 	gint totem_format;
+#if TOTEM_PL_PARSER_CHECK_VERSION(2,29,1)
+	TotemPlPlaylist *playlist;
+	GFile *file;
+#endif
 
 	g_return_if_fail (RB_IS_PLAYLIST_SOURCE (source));
 
 	rb_debug ("saving playlist");
-	playlist = totem_pl_parser_new ();
+	pl = totem_pl_parser_new ();
 
 	g_object_get (source, "name", &name, NULL);
 
@@ -621,11 +686,23 @@ rb_playlist_source_save_playlist (RBPlaylistSource *source,
 		break;
 	}
 
-	totem_pl_parser_write_with_title (playlist, GTK_TREE_MODEL (source->priv->model),
+#if TOTEM_PL_PARSER_CHECK_VERSION(2,29,1)
+	file = g_file_new_for_uri (uri);
+	playlist = totem_pl_playlist_new ();
+
+	gtk_tree_model_foreach (GTK_TREE_MODEL (source->priv->model),
+				(GtkTreeModelForeachFunc)playlist_iter_foreach,
+				playlist);
+	totem_pl_parser_save (pl, playlist, file, name, totem_format, &error);
+	g_object_unref (playlist);
+	g_object_unref (file);
+#else
+	totem_pl_parser_write_with_title (pl, GTK_TREE_MODEL (source->priv->model),
 					  playlist_iter_func, uri, name,
 					  totem_format,
 					  NULL, &error);
-	g_object_unref (playlist);
+#endif
+	g_object_unref (pl);
 	g_free (name);
 	if (error != NULL) {
 		rb_error_dialog (NULL, _("Couldn't save playlist"),
@@ -980,6 +1057,7 @@ rb_playlist_source_mark_dirty (RBPlaylistSource *source)
 
 	klass = RB_PLAYLIST_SOURCE_GET_CLASS (source);
 	klass->impl_mark_dirty (source);
+	g_object_notify (G_OBJECT (source), "dirty");
 }
 
 /**
@@ -987,7 +1065,9 @@ rb_playlist_source_mark_dirty (RBPlaylistSource *source)
  * @source: a #RBPlaylistSource
  * @location: a URI to check
  *
- * Return value: TRUE if the specified URI is in the playlist entry map
+ * Returns TRUE if the specified URI is in the playlist entry map
+ *
+ * Return value: %TRUE if the URI is present
  */
 gboolean
 rb_playlist_source_location_in_map (RBPlaylistSource *source,
@@ -1077,3 +1157,14 @@ rb_playlist_source_make_sorting_key (RBPlaylistSource *source)
 	return sorting_key;
 }
 
+static void
+remove_from_playlist_cmd (GtkAction *action, RBSource *source)
+{
+	rb_source_delete (source);
+}
+
+static char *
+impl_get_delete_action (RBSource *source)
+{
+	return g_strdup ("RemoveFromPlaylist");
+}
