@@ -170,6 +170,7 @@ rb_rating_render_stars (GtkWidget *widget,
 {
 	int i, icon_width;
 	gboolean rtl;
+	cairo_t *cr;
 
 	g_return_val_if_fail (widget != NULL, FALSE);
 	g_return_val_if_fail (window != NULL, FALSE);
@@ -178,6 +179,7 @@ rb_rating_render_stars (GtkWidget *widget,
 	rtl = (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL);
 	gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &icon_width, NULL);
 
+	cr = gdk_cairo_create (window);
 	for (i = 0; i < RB_RATING_MAX_SCORE; i++) {
 		GdkPixbuf *buf;
 		GtkStateType state;
@@ -225,15 +227,11 @@ rb_rating_render_stars (GtkWidget *widget,
 			star_offset = i * icon_width;
 		}
 
-		gdk_draw_pixbuf (window,
-				 NULL,
-				 buf,
-				 x, y,
-				 x_offset + star_offset, y_offset,
-				 icon_width, icon_width,
-				 GDK_RGB_DITHER_NORMAL, 0, 0);
-		g_object_unref (G_OBJECT (buf));
+		gdk_cairo_set_source_pixbuf (cr, buf, x_offset + star_offset, y_offset);
+		cairo_paint (cr);
+		g_object_unref (buf);
 	}
+	cairo_destroy (cr);
 
 	return TRUE;
 }
